@@ -17,8 +17,28 @@ from scipy.fftpack import _fftpack as fftpack
 
 from numpy import arange, add, array, asarray, zeros, dot, exp, pi,\
      swapaxes, double, cdouble
+import numpy as np
 import numpy.fft
 
+# "large" composite numbers supported by FFTPACK
+LARGE_COMPOSITE_SIZES = [
+    2**13,
+    2**5 * 3**5,
+    2**3 * 3**3 * 5**2,
+]
+SMALL_COMPOSITE_SIZES = [
+    2,
+    2*3*5,
+    2*2*3*3,
+]
+# prime
+LARGE_PRIME_SIZES = [
+    2011
+]
+SMALL_PRIME_SIZES = [
+    29
+]
+
 from numpy.random import rand
 def random(size):
     return rand(*size)
@@ -63,7 +83,7 @@ def direct_rdft(x):
     n = len(x)
     w = -arange(n)*(2j*pi/n)
     r = zeros(n,dtype=double)
-    for i in range(n/2+1):
+    for i in range(int(n/2+1)):
         y = dot(exp(i*w),x)
         if i:
             r[2*i-1] = y.real
@@ -77,7 +97,7 @@ def direct_irdft(x):
     x = asarray(x)
     n = len(x)
     x1 = zeros(n,dtype=cdouble)
-    for i in range(n/2+1):
+    for i in range(int(n/2+1)):
         if i:
             if 2*i<n:
                 x1[i] = x[2*i-1] + 1j* x[2*i]
@@ -88,28 +108,37 @@ def direct_irdft(x):
             x1[0] = x[0]
     return direct_idft(x1).real
 
-class TestFft(TestCase):
+class _TestFFTBase(TestCase):
+    def setUp(self):
+        self.cdt = None
+        self.rdt = None
 
     def test_definition(self):
-        x = [1,2,3,4+1j,1,2,3,4+2j]
+        x = np.array([1,2,3,4+1j,1,2,3,4+2j], dtype = self.cdt)
         y = fft(x)
+        self.failUnless(y.dtype == self.cdt,
+                "Output dtype is %s, expected %s" % (y.dtype, self.cdt))
         y1 = direct_dft(x)
         assert_array_almost_equal(y,y1)
-        x = [1,2,3,4+0j,5]
+        x = np.array([1,2,3,4+0j,5], dtype = self.cdt)
         assert_array_almost_equal(fft(x),direct_dft(x))
 
     def test_n_argument_real(self):
-        x1 = [1,2,3,4]
-        x2 =  [1,2,3,4]
+        x1 = np.array([1,2,3,4], dtype=self.rdt)
+        x2 = np.array([1,2,3,4], dtype=self.rdt)
         y = fft([x1,x2],n=4)
+        self.failUnless(y.dtype == self.cdt,
+                "Output dtype is %s, expected %s" % (y.dtype, self.cdt))
         assert_equal(y.shape,(2,4))
         assert_array_almost_equal(y[0],direct_dft(x1))
         assert_array_almost_equal(y[1],direct_dft(x2))
 
     def _test_n_argument_complex(self):
-        x1 = [1,2,3,4+1j]
-        x2 =  [1,2,3,4+1j]
+        x1 = np.array([1,2,3,4+1j], dtype=self.cdt)
+        x2 = np.array([1,2,3,4+1j], dtype=self.cdt)
         y = fft([x1,x2],n=4)
+        self.failUnless(y.dtype == self.cdt,
+                "Output dtype is %s, expected %s" % (y.dtype, self.cdt))
         assert_equal(y.shape,(2,4))
         assert_array_almost_equal(y[0],direct_dft(x1))
         assert_array_almost_equal(y[1],direct_dft(x2))
@@ -124,21 +153,43 @@ class TestFft(TestCase):
             y = fftpack.zrfft(x)
             assert_array_almost_equal(y,y2)
 
+class TestDoubleFFT(_TestFFTBase):
+    def setUp(self):
+        self.cdt = np.cdouble
+        self.rdt = np.double
 
-class TestIfft(TestCase):
+class TestSingleFFT(_TestFFTBase):
+    def setUp(self):
+        self.cdt = np.complex64
+        self.rdt = np.float32
 
+    @dec.knownfailureif(True, "single-precision FFT implementation is partially disabled, until accuracy issues with large prime powers are resolved")
+    def test_notice(self):
+        pass
+
+class _TestIFFTBase(TestCase):
     def test_definition(self):
-        x = [1,2,3,4+1j,1,2,3,4+2j]
+        x = np.array([1,2,3,4+1j,1,2,3,4+2j], self.cdt)
         y = ifft(x)
         y1 = direct_idft(x)
+        self.failUnless(y.dtype == self.cdt,
+                "Output dtype is %s, expected %s" % (y.dtype, self.cdt))
         assert_array_almost_equal(y,y1)
-        x = [1,2,3,4+0j,5]
+
+        x = np.array([1,2,3,4+0j,5], self.cdt)
         assert_array_almost_equal(ifft(x),direct_idft(x))
-        x = [1,2,3,4,1,2,3,4]
+
+    def test_definition_real(self):
+        x = np.array([1,2,3,4,1,2,3,4], self.rdt)
         y = ifft(x)
+        self.failUnless(y.dtype == self.cdt,
+                "Output dtype is %s, expected %s" % (y.dtype, self.cdt))
         y1 = direct_idft(x)
         assert_array_almost_equal(y,y1)
-        x = [1,2,3,4,5]
+
+        x = np.array([1,2,3,4,5], dtype=self.rdt)
+        self.failUnless(y.dtype == self.cdt,
+                "Output dtype is %s, expected %s" % (y.dtype, self.cdt))
         assert_array_almost_equal(ifft(x),direct_idft(x))
 
     def test_djbfft(self):
@@ -153,29 +204,74 @@ class TestIfft(TestCase):
 
     def test_random_complex(self):
         for size in [1,51,111,100,200,64,128,256,1024]:
-            x = random([size]).astype(cdouble)
-            x = random([size]).astype(cdouble) +1j*x
-            assert_array_almost_equal (ifft(fft(x)),x)
-            assert_array_almost_equal (fft(ifft(x)),x)
+            x = random([size]).astype(self.cdt)
+            x = random([size]).astype(self.cdt) +1j*x
+            y1 = ifft(fft(x))
+            y2 = fft(ifft(x))
+            self.failUnless(y1.dtype == self.cdt,
+                    "Output dtype is %s, expected %s" % (y1.dtype, self.cdt))
+            self.failUnless(y2.dtype == self.cdt,
+                    "Output dtype is %s, expected %s" % (y2.dtype, self.cdt))
+            assert_array_almost_equal (y1, x)
+            assert_array_almost_equal (y2, x)
 
     def test_random_real(self):
         for size in [1,51,111,100,200,64,128,256,1024]:
-            x = random([size]).astype(double)
-            assert_array_almost_equal (ifft(fft(x)),x)
-            assert_array_almost_equal (fft(ifft(x)),x)
+            x = random([size]).astype(self.rdt)
+            y1 = ifft(fft(x))
+            y2 = fft(ifft(x))
+            self.failUnless(y1.dtype == self.cdt,
+                    "Output dtype is %s, expected %s" % (y1.dtype, self.cdt))
+            self.failUnless(y2.dtype == self.cdt,
+                    "Output dtype is %s, expected %s" % (y2.dtype, self.cdt))
+            assert_array_almost_equal (y1, x)
+            assert_array_almost_equal (y2, x)
 
+    def test_size_accuracy(self):
+        # Sanity check for the accuracy for prime and non-prime sized inputs
+        if self.rdt == np.float32:
+            rtol = 1e-5
+        elif self.rdt == np.float64:
+            rtol = 1e-10
 
-class TestRfft(TestCase):
+        for size in LARGE_COMPOSITE_SIZES + LARGE_PRIME_SIZES:
+            np.random.seed(1234)
+            x = np.random.rand(size).astype(self.rdt)
+            y = ifft(fft(x))
+            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+                            (size, self.rdt))
+            y = fft(ifft(x))
+            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+                            (size, self.rdt))
+
+            x = (x + 1j*np.random.rand(size)).astype(self.cdt)
+            y = ifft(fft(x))
+            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+                            (size, self.rdt))
+            y = fft(ifft(x))
+            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+                            (size, self.rdt))
+
+class TestDoubleIFFT(_TestIFFTBase):
+    def setUp(self):
+        self.cdt = np.cdouble
+        self.rdt = np.double
+
+class TestSingleIFFT(_TestIFFTBase):
+    def setUp(self):
+        self.cdt = np.complex64
+        self.rdt = np.float32
+
+class _TestRFFTBase(TestCase):
 
     def test_definition(self):
-        x = [1,2,3,4,1,2,3,4]
-        y = rfft(x)
-        y1 = direct_rdft(x)
-        assert_array_almost_equal(y,y1)
-        x = [1,2,3,4,1,2,3,4,5]
-        y = rfft(x)
-        y1 = direct_rdft(x)
-        assert_array_almost_equal(y,y1)
+        for t in [[1, 2, 3, 4, 1, 2, 3, 4], [1, 2, 3, 4, 1, 2, 3, 4, 5]]:
+            x = np.array(t, dtype=self.rdt)
+            y = rfft(x)
+            y1 = direct_rdft(x)
+            assert_array_almost_equal(y,y1)
+            self.failUnless(y.dtype == self.rdt,
+                    "Output dtype is %s, expected %s" % (y.dtype, self.rdt))
 
     def test_djbfft(self):
         from numpy.fft import fft as numpy_fft
@@ -186,28 +282,40 @@ class TestRfft(TestCase):
             y1 = zeros((n,),dtype=double)
             y1[0] = y2[0].real
             y1[-1] = y2[n/2].real
-            for k in range(1,n/2):
+            for k in range(1, int(n/2)):
                 y1[2*k-1] = y2[k].real
                 y1[2*k] = y2[k].imag
             y = fftpack.drfft(x)
             assert_array_almost_equal(y,y1)
 
+class TestRFFTDouble(_TestRFFTBase):
+    def setUp(self):
+        self.cdt = np.cdouble
+        self.rdt = np.double
 
-class TestIrfft(TestCase):
+class TestRFFTSingle(_TestRFFTBase):
+    def setUp(self):
+        self.cdt = np.complex64
+        self.rdt = np.float32
+
+class _TestIRFFTBase(TestCase):
 
     def test_definition(self):
-        x = [1,2,3,4,1,2,3,4]
-        x1 = [1,2+3j,4+1j,2+3j,4,2-3j,4-1j,2-3j]
-        y = irfft(x)
-        y1 = direct_irdft(x)
-        assert_array_almost_equal(y,y1)
-        assert_array_almost_equal(y,ifft(x1))
-        x = [1,2,3,4,1,2,3,4,5]
-        x1 = [1,2+3j,4+1j,2+3j,4+5j,4-5j,2-3j,4-1j,2-3j]
-        y = irfft(x)
-        y1 = direct_irdft(x)
-        assert_array_almost_equal(y,y1)
-        assert_array_almost_equal(y,ifft(x1))
+        x1 = [1,2,3,4,1,2,3,4]
+        x1_1 = [1,2+3j,4+1j,2+3j,4,2-3j,4-1j,2-3j]
+        x2= [1,2,3,4,1,2,3,4,5]
+        x2_1 = [1,2+3j,4+1j,2+3j,4+5j,4-5j,2-3j,4-1j,2-3j]
+
+        def _test(x, xr):
+            y = irfft(np.array(x, dtype=self.rdt))
+            y1 = direct_irdft(x)
+            self.failUnless(y.dtype == self.rdt,
+                    "Output dtype is %s, expected %s" % (y.dtype, self.rdt))
+            assert_array_almost_equal(y,y1, decimal=self.ndec)
+            assert_array_almost_equal(y,ifft(xr), decimal=self.ndec)
+
+        _test(x1, x1_1)
+        _test(x2, x2_1)
 
     def test_djbfft(self):
         from numpy.fft import ifft as numpy_ifft
@@ -216,7 +324,7 @@ class TestIrfft(TestCase):
             x = range(n)
             x1 = zeros((n,),dtype=cdouble)
             x1[0] = x[0]
-            for k in range(1,n/2):
+            for k in range(1, int(n/2)):
                 x1[k] = x[2*k-1]+1j*x[2*k]
                 x1[n-k] = x[2*k-1]-1j*x[2*k]
             x1[n/2] = x[-1]
@@ -226,9 +334,48 @@ class TestIrfft(TestCase):
 
     def test_random_real(self):
         for size in [1,51,111,100,200,64,128,256,1024]:
-            x = random([size]).astype(double)
-            assert_array_almost_equal (irfft(rfft(x)),x)
-            assert_array_almost_equal (rfft(irfft(x)),x)
+            x = random([size]).astype(self.rdt)
+            y1 = irfft(rfft(x))
+            y2 = rfft(irfft(x))
+            self.failUnless(y1.dtype == self.rdt,
+                    "Output dtype is %s, expected %s" % (y1.dtype, self.rdt))
+            self.failUnless(y2.dtype == self.rdt,
+                    "Output dtype is %s, expected %s" % (y2.dtype, self.rdt))
+            assert_array_almost_equal (y1, x, decimal=self.ndec,
+                                       err_msg="size=%d" % size)
+            assert_array_almost_equal (y2, x, decimal=self.ndec,
+                                       err_msg="size=%d" % size)
+
+    def test_size_accuracy(self):
+        # Sanity check for the accuracy for prime and non-prime sized inputs
+        if self.rdt == np.float32:
+            rtol = 1e-5
+        elif self.rdt == np.float64:
+            rtol = 1e-10
+
+        for size in LARGE_COMPOSITE_SIZES + LARGE_PRIME_SIZES:
+            np.random.seed(1234)
+            x = np.random.rand(size).astype(self.rdt)
+            y = irfft(rfft(x))
+            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+                            (size, self.rdt))
+            y = rfft(irfft(x))
+            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+                            (size, self.rdt))
+
+# self.ndec is bogus; we should have a assert_array_approx_equal for number of
+# significant digits
+class TestIRFFTDouble(_TestIRFFTBase):
+    def setUp(self):
+        self.cdt = np.cdouble
+        self.rdt = np.double
+        self.ndec = 14
+
+class TestIRFFTSingle(_TestIRFFTBase):
+    def setUp(self):
+        self.cdt = np.complex64
+        self.rdt = np.float32
+        self.ndec = 5
 
 class Testfft2(TestCase):
     def test_regression_244(self):
@@ -240,6 +387,35 @@ class Testfft2(TestCase):
         y_r = numpy.fft.fftn(x, s=(8, 8), axes=(-3,  -2))
         assert_array_almost_equal(y, y_r)
 
+class TestFftnSingle(TestCase):
+    def test_definition(self):
+        x = [[1,2,3],[4,5,6],[7,8,9]]
+        y = fftn(np.array(x, np.float32))
+        if not y.dtype == np.complex64:
+            raise ValueError("double precision output with single precision")
+
+        y_r = np.array(fftn(x), np.complex64)
+        assert_array_almost_equal_nulp(y, y_r)
+
+    def test_size_accuracy(self):
+        for size in SMALL_COMPOSITE_SIZES + SMALL_PRIME_SIZES:
+            np.random.seed(1234)
+            x = np.random.rand(size, size) + 1j*np.random.rand(size, size)
+            y1 = fftn(x.astype(np.float32))
+            y2 = fftn(x.astype(np.float64)).astype(np.complex64)
+
+            self.failUnless(y1.dtype == np.complex64)
+            assert_array_almost_equal_nulp(y1, y2, 2000)
+
+        for size in LARGE_COMPOSITE_SIZES + LARGE_PRIME_SIZES:
+            np.random.seed(1234)
+            x = np.random.rand(size, 3) + 1j*np.random.rand(size, 3)
+            y1 = fftn(x.astype(np.float32))
+            y2 = fftn(x.astype(np.float64)).astype(np.complex64)
+
+            self.failUnless(y1.dtype == np.complex64)
+            assert_array_almost_equal_nulp(y1, y2, 2000)
+
 class TestFftn(TestCase):
 
     def test_definition(self):
@@ -395,22 +571,67 @@ class TestFftn(TestCase):
         except ValueError:
             pass
 
-class TestIfftn(TestCase):
-
+class _TestIfftn(TestCase):
+    dtype = None
+    cdtype = None
     def test_definition(self):
-        x = [[1,2,3],[4,5,6],[7,8,9]]
+        x = np.array([[1,2,3],[4,5,6],[7,8,9]], dtype=self.dtype)
         y = ifftn(x)
-        assert_array_almost_equal(y,direct_idftn(x))
+        assert y.dtype == self.cdtype
+        assert_array_almost_equal_nulp(y,direct_idftn(x),self.maxnlp)
         x = random((20,26))
-        assert_array_almost_equal(ifftn(x),direct_idftn(x))
+        assert_array_almost_equal_nulp(ifftn(x),direct_idftn(x),self.maxnlp)
         x = random((5,4,3,20))
-        assert_array_almost_equal(ifftn(x),direct_idftn(x))
+        assert_array_almost_equal_nulp(ifftn(x),direct_idftn(x),self.maxnlp)
 
     def test_random_complex(self):
         for size in [1,2,51,32,64,92]:
             x = random([size,size]) + 1j*random([size,size])
-            assert_array_almost_equal (ifftn(fftn(x)),x)
-            assert_array_almost_equal (fftn(ifftn(x)),x)
+            assert_array_almost_equal_nulp(ifftn(fftn(x)),x,self.maxnlp)
+            assert_array_almost_equal_nulp(fftn(ifftn(x)),x,self.maxnlp)
+
+class TestIfftnDouble(_TestIfftn):
+    dtype = np.float64
+    cdtype = np.complex128
+    maxnlp = 2000
+
+class TestIfftnSingle(_TestIfftn):
+    dtype = np.float32
+    cdtype = np.complex64
+    maxnlp = 3500
+
+class TestLongDoubleFailure(TestCase):
+
+    def test_complex(self):
+        if np.dtype(np.longcomplex).itemsize == np.dtype(np.complex).itemsize:
+            # longdouble == double; so fft is supported
+            return
+
+        x = np.random.randn(10).astype(np.longdouble) + \
+                1j * np.random.randn(10).astype(np.longdouble)
+
+        for f in [fft, ifft]:
+            try:
+                f(x)
+                raise AssertionError("Type %r not supported but does not fail" % \
+                                     np.longcomplex)
+            except ValueError:
+                pass
+
+    def test_real(self):
+        if np.dtype(np.longdouble).itemsize == np.dtype(np.double).itemsize:
+            # longdouble == double; so fft is supported
+            return
+
+        x = np.random.randn(10).astype(np.longcomplex)
+
+        for f in [fft, ifft]:
+            try:
+                f(x)
+                raise AssertionError("Type %r not supported but does not fail" % \
+                                     np.longcomplex)
+            except ValueError:
+                pass
 
 if __name__ == "__main__":
     run_module_suite()
