@@ -77,16 +77,17 @@ def freqs(b,a,worN=None,plot=None):
     return w, h
 
 def freqz(b, a=1, worN=None, whole=0, plot=None):
-    """Compute frequency response of a digital filter.
+    """
+    Compute the frequency response of a digital filter.
 
-    Given the numerator (b) and denominator (a) of a digital filter compute
-    its frequency response.
+    Given the numerator ``b`` and denominator ``a`` of a digital filter compute
+    its frequency response::
 
                jw               -jw            -jmw
         jw  B(e)    b[0] + b[1]e + .... + b[m]e
      H(e) = ---- = ------------------------------------
                jw               -jw            -jnw
-            A(e)    a[0] + a[2]e + .... + a[n]e
+            A(e)    a[0] + a[1]e + .... + a[n]e
 
     Parameters
     ----------
@@ -95,10 +96,9 @@ def freqz(b, a=1, worN=None, whole=0, plot=None):
     a : ndarray
         numerator of a linear filter
     worN : {None, int}, optional
-        If None, then compute at 200 frequencies around the interesting parts
-        of the response curve (determined by pole-zero locations).  If a single
-        integer, the compute at that many frequencies.  Otherwise, compute the
-        response at frequencies given in worN.
+        If None, then compute at 512 frequencies around the unit circle.
+        If a single integer, the compute at that many frequencies.
+        Otherwise, compute the response at frequencies given in worN
     whole : {0,1}, optional
         Normally, frequencies are computed from 0 to pi (upper-half of
         unit-circle.  If whole is non-zero compute frequencies from 0 to 2*pi.
@@ -109,6 +109,30 @@ def freqz(b, a=1, worN=None, whole=0, plot=None):
         The frequencies at which h was computed.
     h : ndarray
         The frequency response.
+
+    Examples
+    --------
+
+    >>> b = firwin(80, 0.5, window=('kaiser', 8))
+    >>> h, w = freqz(b)
+
+    >>> import matplotlib.pyplot as plt
+    >>> fig = plt.figure()
+    >>> plt.title('Digital filter frequency response')
+    >>> ax1 = fig.add_subplot(111)
+
+    >>> plt.semilogy(h, np.abs(w), 'b')
+    >>> plt.ylabel('Amplitude (dB)', color='b')
+    >>> plt.xlabel('Frequency (rad/sample)')
+    >>> plt.grid()
+    >>> plt.legend()
+
+    >>> ax2 = ax1.twinx()
+    >>> angles = np.unwrap(np.angle(w))
+    >>> plt.plot(h, angles, 'g')
+    >>> plt.ylabel('Angle (radians)', color='g')
+    >>> plt.show()
+
     """
     b, a = map(atleast_1d, (b,a))
     if whole:
@@ -216,15 +240,15 @@ def normalize(b,a):
         b = asarray([b],b.dtype.char)
     while a[0] == 0.0 and len(a) > 1:
         a = a[1:]
-    if allclose(b[:,0], 0, rtol=1e-14):
-        warnings.warn("Badly conditionned filter coefficients (numerator): the "
-                      "results may be meaningless", BadCoefficients)
-        while allclose(b[:,0], 0, rtol=1e-14) and (b.shape[-1] > 1):
-            b = b[:,1:]
-    if b.shape[0] == 1:
-        b = b[0]
     outb = b * (1.0) / a[0]
     outa = a * (1.0) / a[0]
+    if allclose(outb[:,0], 0, rtol=1e-14):
+        warnings.warn("Badly conditioned filter coefficients (numerator): the "
+                      "results may be meaningless", BadCoefficients)
+        while allclose(outb[:,0], 0, rtol=1e-14) and (outb.shape[-1] > 1):
+            outb = outb[:,1:]
+    if outb.shape[0] == 1:
+        outb = outb[0]
     return outb, outa
 
 
@@ -1551,7 +1575,8 @@ def kaiserord(ripple, width):
     return ceil(N), beta
 
 def firwin(N, cutoff, width=None, window='hamming'):
-    """FIR Filter Design using windowed ideal filter method.
+    """
+    FIR Filter Design using windowed ideal filter method.
 
     Parameters
     ----------
@@ -1562,11 +1587,13 @@ def firwin(N, cutoff, width=None, window='hamming'):
     width  -- if width is not None, then assume it is the approximate width of
               the transition region (normalized so that 1 corresonds to pi)
               for use in kaiser FIR filter design.
-    window -- desired window to use.
+    window -- desired window to use. See get_window for a list
+              of windows and required parameters.
 
     Returns
     -------
     h      -- coefficients of length N fir filter.
+
     """
 
     from signaltools import get_window
